package lightcontainer.repository;

import lightcontainer.domains.client.StoreProcessor;
import lightcontainer.domains.Task;
import lightcontainer.interfaces.ClientHandlerFFE;
import lightcontainer.interfaces.ProtocolRepository;
import lightcontainer.interfaces.StoreProcessorFFE;
import lightcontainer.protocol.ProtocolWriter;

import java.util.Deque;
import java.util.Iterator;
import java.util.concurrent.ConcurrentLinkedDeque;

public class FileFrontEnd implements ClientHandlerFFE, StoreProcessorFFE {
    // Variables
    private Deque<Task> tasks = new ConcurrentLinkedDeque<>();
    private ClientHandlerRepository clientRepository; // TODO -> pourquoi pas une interface ? end
    private StoreProcessorRepository storeRepository; // TODO -> pourquoi pas une interface ? end
    private ProtocolRepository protocolRepository;

    // Constructor
    public FileFrontEnd(ClientHandlerRepository clientRepo, StoreProcessorRepository storeRepo, ProtocolRepository protocolRepository) {
        this.clientRepository = clientRepo;
        this.storeRepository = storeRepo;
        this.protocolRepository = protocolRepository;
    }

    /**
     * Appelé quand nouvelle tâche
     */
    public void alertStoreProcessors(Task task) {
        // On avertit les stor processors d'une nouvelle tâche

    }

    /**
     * Permet à un {@link StoreProcessor} d'avertir le FFE qu'il est disponible
     *
     * @param store Le SBE qui s'est occupé de la tâche
     * @param response La réponse à envoyer au client
     */
    @Override
    public void onStoreAvailable(StoreProcessor store, ProtocolWriter.ProtocolResult response) {
        // TODO : Chercher une tâche appropriée
        Iterator<Task> it = tasks.iterator();

        while (it.hasNext()) {
            Task task = it.next();
            if (task.isResponseOfClient(store.getName())) {
                clientRepository.respondToClient(task.getClient(), response);
                it.remove(); // Suppression de la tâche
            }
        }
    }

    @Override
    public void newCommand(String command, String client) {
        // TODO : Ajouter la tâche - Alerter les StorBackEnds
        Task task = Task.newInstance(command, client);
        tasks.add(task);
    }
}
